//===========================================================================
//  @(#) $Name: arts++-1-1-a12 $
//  @(#) $Id: ArtsNextHopTableData.hh,v 1.2 2004/04/21 23:51:27 kkeys Exp $
//===========================================================================
//  Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software.  You can redistribute it
//  and/or modify it under the terms of the GNU Lesser General Public
//  License, Version 2.1, February 1999, which is incorporated by
//  reference herein.
//
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual
//  property rights.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with arts++.  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/lesser.html
//
//  or by writing to:
//
//  Free Software Foundation, Inc.
//  59 Temple Place, Suite 330
//  Boston, MA 02111-1307
//  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#ifndef _ARTSNEXTHOPTABLEDATA_HH_
#define _ARTSNEXTHOPTABLEDATA_HH_

extern "C" 
{
#include <sys/types.h>

#include "caida_t.h"
}

#include <vector>

#include "ArtsNextHopTableEntry.hh"

//---------------------------------------------------------------------------
//  class ArtsNextHopTableData
//---------------------------------------------------------------------------
//!  This class encapsulates the data portion of an ARTS NextHop table
//!  object.  It contains a vector of NextHops entries, each of which
//!  contains an IP address plus packet and byte counters for traffic
//!  using the NextHop.
//---------------------------------------------------------------------------
class ArtsNextHopTableData
{
public:
  //-------------------------------------------------------------------------
  //                           ArtsNextHopTableData()
  //.........................................................................
  //!  constructor
  //-------------------------------------------------------------------------
  ArtsNextHopTableData();
  
  //-------------------------------------------------------------------------
  //                          ~ArtsNextHopTableData()
  //.........................................................................
  //!  destructor
  //-------------------------------------------------------------------------
  ~ArtsNextHopTableData();
  
  //-------------------------------------------------------------------------
  //                     uint16_t SampleInterval() const
  //.........................................................................
  //!  Returns the sampling ratio, indicating the 1:N packet sampling
  //!  used to obtain the data.  Normally this value is 1, meaning we're
  //!  working with complete data sets (for example, from Cisco
  //!  flow-export).
  //-------------------------------------------------------------------------
  uint16_t SampleInterval() const;
  
  //-------------------------------------------------------------------------
  //            uint16_t SampleInterval(uint16_t sampleInterval)
  //.........................................................................
  //!  Sets and returns the sampling ratio, indicating the 1:N packet
  //!  sampling used to obtain the data.  Normally this value is 1,
  //!  meaning we're working with complete data sets (for example, from
  //!  Cisco flow-export).
  //-------------------------------------------------------------------------
  uint16_t SampleInterval(uint16_t sampleInterval);
  
  //-------------------------------------------------------------------------
  //                       uint64_t TotalPkts() const
  //.........................................................................
  //!  Returns the total packets of traffic in the ArtsNextHopTableData
  //!  object.  The total is the sum of all traffic for all NextHops
  //!  contained in the object.
  //-------------------------------------------------------------------------
  uint64_t TotalPkts() const;
  
  //-------------------------------------------------------------------------
  //                 uint64_t TotalPkts(uint64_t totalPkts)
  //.........................................................................
  //!  Sets and returns the total packets of traffic in the
  //!  ArtsNextHopTableData object.  The total is the sum of all traffic
  //!  for all NextHops contained in the object.
  //-------------------------------------------------------------------------
  uint64_t TotalPkts(uint64_t totalPkts);
  
  //-------------------------------------------------------------------------
  //                       uint64_t TotalBytes() const
  //.........................................................................
  //!  Returns the total bytes of traffic in the ArtsNextHopTableData
  //!  object.  The total is the sum of all traffic for all NextHops
  //!  contained in the object.
  //-------------------------------------------------------------------------
  uint64_t TotalBytes() const;
  
  //-------------------------------------------------------------------------
  //                uint64_t TotalBytes(uint64_t totalBytes)
  //.........................................................................
  //!  Sets and returns the total bytes of traffic in the
  //!  ArtsNextHopTableData object.  The total is the sum of all traffic
  //!  for all NextHops contained in the object.
  //-------------------------------------------------------------------------
  uint64_t TotalBytes(uint64_t totalBytes);
  
  //-------------------------------------------------------------------------
  //            std::vector<ArtsNextHopTableEntry> & NextHopEntries() const
  //.........................................................................
  //!  Returns a reference to the vector of NextHop entries in the
  //!  ArtsNextHopTableData object.
  //-------------------------------------------------------------------------
  std::vector<ArtsNextHopTableEntry> & NextHopEntries() const;

  //-------------------------------------------------------------------------
  //                        void SortEntriesByBytes()
  //.........................................................................
  //!  Sorts the contained ArtsNextHopTableEntry objects in descending
  //!  order by bytes of traffic.
  //-------------------------------------------------------------------------
  void SortEntriesByBytes();

  //--------------------------------------------------------------------------
  //       void SortEntriesByPkts() 
  //..........................................................................
  //!  Sorts the contained ArtsNextHopTableEntry objects in descending
  //!  order by packets of traffic.
  //--------------------------------------------------------------------------
  void SortEntriesByPkts();
  
  //-------------------------------------------------------------------------
  //            uint32_t ComputeLength(uint8_t version = 0) const
  //.........................................................................
  //!  Computes the bytes required to store the ArtsNextHopTableData
  //!  object in a file.
  //-------------------------------------------------------------------------
  uint32_t ComputeLength(uint8_t version = 0) const;
  
  //-------------------------------------------------------------------------
  //                uint32_t Length(uint8_t version = 0) const
  //.........................................................................
  //!  Returns the bytes required to store the ArtsNextHopTableData
  //!  object in a file.
  //-------------------------------------------------------------------------
  uint32_t Length(uint8_t version = 0) const;
  
  //-------------------------------------------------------------------------
  //           std::istream& read(std::istream& is, uint8_t version = 0)
  //.........................................................................
  //!  Reads the contents of an ArtsNextHopTableData from an istream.
  //!  Returns the istream.
  //-------------------------------------------------------------------------
  std::istream& read(std::istream& is, uint8_t version = 0);
  
  //-------------------------------------------------------------------------
  //                 int read(int fd, uint8_t version = 0)
  //.........................................................................
  //!  Reads the contents of an ArtsNextHopTableData from a file
  //!  descriptor.  Returns the number of bytes read if successful, -1
  //!  on failure.
  //-------------------------------------------------------------------------
  int read(int fd, uint8_t version = 0);
  
  //-------------------------------------------------------------------------
  //          std::ostream& write(std::ostream& os, uint8_t version = 0) const
  //.........................................................................
  //!  Writes the contents of an ArtsNextHopTableData to an ostream.
  //!  Returns the ostream.
  //-------------------------------------------------------------------------
  std::ostream& write(std::ostream& os, uint8_t version = 0) const;
  
  //-------------------------------------------------------------------------
  //              int write(int fd, uint8_t version = 0) const
  //.........................................................................
  //!  Writes the contents of an ArtsNextHopTableData to a file
  //!  descriptor.  Returns the number of bytes written on success, -1
  //!  on failure.
  //-------------------------------------------------------------------------
  int write(int fd, uint8_t version = 0) const;

  //-------------------------------------------------------------------------
  //    ArtsNextHopTableData &
  //    operator = (const ArtsNextHopTableData & artsNextHopTableData)
  //.........................................................................
  //!  Overloaded '=' operator for assigning one ArtsNextHopTableData
  //!  to another.
  //-------------------------------------------------------------------------
  ArtsNextHopTableData &
  operator = (const ArtsNextHopTableData & artsNextHopTableData);

  //-------------------------------------------------------------------------
  //    friend std::ostream &                            
  //    operator << (std::ostream& os,                   
  //                 const ArtsNextHopTableData & artsNextHopTableData)
  //.........................................................................
  //!  Overload ostream '<<' operator to dump the contents of an
  //!  ArtsNextHopTableData object to an ostream in a human-readable
  //!  form.  Returns the ostream.
  //-------------------------------------------------------------------------
  friend std::ostream &
  operator << (std::ostream& os,
               const ArtsNextHopTableData & artsNextHopTableData);

  #ifndef NDEBUG
    //------------------------------------------------------------------------
    //                       static uint32_t NumObjects()                    
    //........................................................................
    //  
    //------------------------------------------------------------------------
    static uint32_t NumObjects()
    {
      return(_numObjects);
    }
  #endif  // NDEBUG

private:
  uint16_t                                _sampleInterval;
  uint64_t                                _totpkts;
  uint64_t                                _totbytes;
  mutable std::vector<ArtsNextHopTableEntry>   _NextHopEntries;
  mutable uint32_t                        _length;

  static uint32_t                         _numObjects;
};

#endif  // _ARTSNEXTHOPTABLEDATA_HH_
